@@ -644,25 +644,27 @@ namespace qc {
             // measure all qubits
             std::map<std::string, std::size_t> counts{};
             for (std::size_t i = 0; i < shots; ++i) {
+                // measure all returns a string of the form "q(n-1) ... q(0)"
                 auto measurement = dd->measureAll(e, false, mt);
                 // reverse the order of the bits so that measurements follow big-endian convention
-                std::reverse(measurement.begin(), measurement.end());
                 counts[measurement]++;
             }
-            std::map<std::string, std::size_t> actualCounts{};
+            // reduce reference count of measured state
+            dd->decRef(e);
 
+            std::map<std::string, std::size_t> actualCounts{};
             for (const auto& [bitstring, count]: counts) {
                 std::string measurement(nclassics, '0');
                 if (hasMeasurements) {
                     // if the circuit contains measurements, we only want to return the measured bits
                     for (const auto& [qubit, bit]: measurementMap) {
                         // measurement map specifies that the circuit `qubit` is measured into a certain `bit`
-                        measurement[bit] = bitstring[qubit];
+                        measurement[nclassics - 1 - bit] = bitstring[bitstring.size() - 1 - qubit];
                     }
                 } else {
                     // otherwise, we consider the output permutation for determining where to measure the qubits to
                     for (const auto& [qubit, bit]: outputPermutation) {
-                        measurement[bit] = bitstring[qubit];
+                        measurement[nclassics - 1 - bit] = bitstring[bitstring.size() - 1 - qubit];
                     }
                 }
                 actualCounts[measurement] += count;
@@ -710,10 +712,12 @@ namespace qc {
                         auto*       classicControlled = dynamic_cast<ClassicControlledOperation*>(op.get());
                         const auto& controlRegister   = classicControlled->getControlRegister();
                         const auto& expectedValue     = classicControlled->getExpectedValue();
-                        auto        actualValue       = 0U;
+                        auto        actualValue       = 0ULL;
                         // determine the actual value from measurements
                         for (std::size_t j = 0; j < controlRegister.second; ++j) {
-                            actualValue |= (measurements[controlRegister.first + j] == '1') ? (1U << j) : 0U;
+                            if (measurements[controlRegister.first + j] == '1') {
+                                actualValue |= 1ULL << j;
+                            }
                         }
 
                         // do not apply an operation if the value is not the expected one
@@ -729,9 +733,8 @@ namespace qc {
                     dd->garbageCollect();
                 }
 
-                // correct permutation if necessary
-                changePermutation(e, permutation, outputPermutation, dd);
-                e = dd->reduceGarbage(e, garbage);
+                // reduce reference count of measured state
+                dd->decRef(e);
 
                 std::string shot(nclassics, '0');
                 for (const auto& [bit, value]: measurements) {
@@ -744,18 +747,13 @@ namespace qc {
         }
     }
 
-    void QuantumComputation::extractProbabilityVector(const VectorDD& in, std::vector<dd::fp>& probVector, std::unique_ptr<dd::Package>& dd) {
+    void QuantumComputation::extractProbabilityVector(const VectorDD& in, dd::ProbabilityVector& probVector, std::unique_ptr<dd::Package>& dd) {
         // ! initial layout, output permutation and garbage qubits are currently not supported here
-
         dd->incRef(in);
-        std::map<std::size_t, char> measurements{};
-        // the number of classics dictates the size of the resulting probability vector
-        probVector = std::vector<dd::fp>(1ULL << nclassics);
-
-        extractProbabilityVectorRecursive(in, ops.begin(), measurements, 1., probVector, dd);
+        extractProbabilityVectorRecursive(in, ops.begin(), std::map<std::size_t, char>{}, 1., probVector, dd);
     }
 
-    void QuantumComputation::extractProbabilityVectorRecursive(const VectorDD& currentState, decltype(ops.begin()) currentIt, std::map<std::size_t, char> measurements, dd::fp commonFactor, std::vector<dd::fp>& probVector, std::unique_ptr<dd::Package>& dd) {
+    void QuantumComputation::extractProbabilityVectorRecursive(const VectorDD& currentState, decltype(ops.begin()) currentIt, std::map<std::size_t, char> measurements, dd::fp commonFactor, dd::ProbabilityVector& probVector, std::unique_ptr<dd::Package>& dd) {
         auto state = currentState;
         for (auto it = currentIt; it != ops.end(); ++it) {
             auto& op = (*it);
@@ -768,7 +766,7 @@ namespace qc {
                 auto        actualValue       = 0U;
                 // determine the actual value from measurements
                 for (std::size_t j = 0; j < controlRegister.second; ++j) {
-                    actualValue |= (measurements[controlRegister.first + j] == '1') ? (1U << j) : 0U;
+                    actualValue |= (measurements[controlRegister.first + j] == '1') ? (1ULL << j) : 0U;
                 }
 
                 // do not apply an operation if the value is not the expected one
@@ -835,7 +833,7 @@ namespace qc {
                     for (std::size_t i = 0; i < nclassics; ++i) {
                         // if this is the qubit being measured and the result is one
                         if (i == static_cast<std::size_t>(classics[0])) {
-                            idx1 |= (1 << i);
+                            idx1 |= (1ULL << i);
                         } else {
                             // sanity check
                             auto findIt = measurements.find(i);
@@ -844,13 +842,20 @@ namespace qc {
                             }
                             // if i-th bit is set increase the index appropriately
                             if (findIt->second == '1') {
-                                idx0 |= (1 << i);
-                                idx1 |= (1 << i);
+                                idx0 |= (1ULL << i);
+                                idx1 |= (1ULL << i);
                             }
                         }
                     }
-                    probVector.at(idx0) = commonFactor * pzero;
-                    probVector.at(idx1) = commonFactor * pone;
+                    const auto prob0 = commonFactor * pzero;
+                    if (!dd::ComplexTable<>::Entry::approximatelyZero(prob0)) {
+                        probVector[idx0] = prob0;
+                    }
+                    const auto prob1 = commonFactor * pone;
+                    if (!dd::ComplexTable<>::Entry::approximatelyZero(prob1)) {
+                        probVector[idx1] = prob1;
+                    }
+
                     // probabilities have been written -> this path is done
                     dd->decRef(state);
                     return;
@@ -1223,8 +1228,8 @@ namespace qc {
         }
     }
 
-    bool QuantumComputation::isIdleQubit(dd::Qubit physical_qubit) const {
-        return !std::any_of(ops.cbegin(), ops.cend(), [&physical_qubit](const auto& op) { return op->actsOn(physical_qubit); });
+    bool QuantumComputation::isIdleQubit(dd::Qubit physicalQubit) const {
+        return !std::any_of(ops.cbegin(), ops.cend(), [&physicalQubit](const auto& op) { return op->actsOn(physicalQubit); });
     }
 
     void QuantumComputation::stripIdleQubits(bool force, bool reduceIOpermutations) {
@@ -1259,59 +1264,59 @@ namespace qc {
         }
     }
 
-    std::string QuantumComputation::getQubitRegister(dd::Qubit physical_qubit_index) const {
+    std::string QuantumComputation::getQubitRegister(dd::Qubit physicalQubitIndex) const {
         for (const auto& reg: qregs) {
             auto start_idx = reg.second.first;
             auto count     = reg.second.second;
-            if (physical_qubit_index < start_idx) continue;
-            if (physical_qubit_index >= start_idx + count) continue;
+            if (physicalQubitIndex < start_idx) continue;
+            if (physicalQubitIndex >= start_idx + count) continue;
             return reg.first;
         }
         for (const auto& reg: ancregs) {
             auto start_idx = reg.second.first;
             auto count     = reg.second.second;
-            if (physical_qubit_index < start_idx) continue;
-            if (physical_qubit_index >= start_idx + count) continue;
+            if (physicalQubitIndex < start_idx) continue;
+            if (physicalQubitIndex >= start_idx + count) continue;
             return reg.first;
         }
 
-        throw QFRException("[getQubitRegister] Qubit index " + std::to_string(physical_qubit_index) + " not found in any register");
+        throw QFRException("[getQubitRegister] Qubit index " + std::to_string(physicalQubitIndex) + " not found in any register");
     }
 
-    std::pair<std::string, dd::Qubit> QuantumComputation::getQubitRegisterAndIndex(dd::Qubit physical_qubit_index) const {
-        std::string reg_name = getQubitRegister(physical_qubit_index);
+    std::pair<std::string, dd::Qubit> QuantumComputation::getQubitRegisterAndIndex(dd::Qubit physicalQubitIndex) const {
+        std::string reg_name = getQubitRegister(physicalQubitIndex);
         dd::Qubit   index    = 0;
         auto        it       = qregs.find(reg_name);
         if (it != qregs.end()) {
-            index = static_cast<dd::Qubit>(physical_qubit_index - it->second.first);
+            index = static_cast<dd::Qubit>(physicalQubitIndex - it->second.first);
         } else {
             auto it_anc = ancregs.find(reg_name);
             if (it_anc != ancregs.end()) {
-                index = static_cast<dd::Qubit>(physical_qubit_index - it_anc->second.first);
+                index = static_cast<dd::Qubit>(physicalQubitIndex - it_anc->second.first);
             }
-            // no else branch needed here, since error would have already shown in getQubitRegister(physical_qubit_index)
+            // no else branch needed here, since error would have already shown in getQubitRegister(physicalQubitIndex)
         }
         return {reg_name, index};
     }
 
-    std::string QuantumComputation::getClassicalRegister(std::size_t classical_index) const {
+    std::string QuantumComputation::getClassicalRegister(std::size_t classicalIndex) const {
         for (const auto& reg: cregs) {
             auto start_idx = reg.second.first;
             auto count     = reg.second.second;
-            if (classical_index < start_idx) continue;
-            if (classical_index >= start_idx + count) continue;
+            if (classicalIndex < start_idx) continue;
+            if (classicalIndex >= start_idx + count) continue;
             return reg.first;
         }
 
-        throw QFRException("[getClassicalRegister] Classical index " + std::to_string(classical_index) + " not found in any register");
+        throw QFRException("[getClassicalRegister] Classical index " + std::to_string(classicalIndex) + " not found in any register");
     }
 
-    std::pair<std::string, std::size_t> QuantumComputation::getClassicalRegisterAndIndex(std::size_t classical_index) const {
-        std::string reg_name = getClassicalRegister(classical_index);
+    std::pair<std::string, std::size_t> QuantumComputation::getClassicalRegisterAndIndex(std::size_t classicalIndex) const {
+        std::string reg_name = getClassicalRegister(classicalIndex);
         std::size_t index    = 0;
         auto        it       = cregs.find(reg_name);
         if (it != cregs.end()) {
-            index = classical_index - it->second.first;
+            index = classicalIndex - it->second.first;
         } // else branch not needed since getClassicalRegister already covers this case
         return {reg_name, index};
     }
@@ -1361,8 +1366,19 @@ namespace qc {
         return max_index;
     }
 
-    bool QuantumComputation::physicalQubitIsAncillary(dd::Qubit physical_qubit_index) const {
-        return std::any_of(ancregs.cbegin(), ancregs.cend(), [&physical_qubit_index](const auto& ancreg) { return ancreg.second.first <= physical_qubit_index && physical_qubit_index < ancreg.second.first + ancreg.second.second; });
+    bool QuantumComputation::physicalQubitIsAncillary(dd::Qubit physicalQubitIndex) const {
+        return std::any_of(ancregs.cbegin(), ancregs.cend(), [&physicalQubitIndex](const auto& ancreg) { return ancreg.second.first <= physicalQubitIndex && physicalQubitIndex < ancreg.second.first + ancreg.second.second; });
+    }
+
+    void QuantumComputation::setLogicalQubitGarbage(dd::Qubit logicalQubitIndex) {
+        garbage[logicalQubitIndex] = true;
+        // setting a logical qubit garbage also means removing it from the output permutation if it was present before
+        for (auto it = outputPermutation.begin(); it != outputPermutation.end(); ++it) {
+            if (it->second == logicalQubitIndex) {
+                outputPermutation.erase(it);
+                break;
+            }
+        }
     }
 
     bool QuantumComputation::isLastOperationOnQubit(const decltype(ops.cbegin())& opIt, const decltype(ops.cend())& end) const {
